#!/usr/bin/perl -w

#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


package Nmap::Scan;

use strict;
use DBI;
use Nmap::Functions;


use constant
{
	TYPE_SINGLE				=> 1,
	TYPE_SCHEDULED			=> 2,
	TYPE_PERIODIC			=> 3,
	TYPE_INHERITED			=> 4,
	TYPE_PERIODICSTOPPED	=> 5,

	OPT_NEW		=> 0,
	OPT_UPDATE	=> 1,
	OPT_DELETE  => 2,
	OPT_NORMAL  => 3,

	STATE_NEW			=> 0,
	STATE_VALIDATED		=> 10,
	STATE_INPROGRESS	=> 20,
	STATE_FINISHED		=> 30,
	STATE_EXECERROR		=> 40,
	STATE_WIP			=> 50,

	OPTIONS		=>  [	'SCAN-TCP'  , 
						'TCP-METHOD'  , 
						'SCAN-UDP' , 
						'PING-TARGET' , 
						'OSDETECT' , 
						'OSDETECT-AGRESSIVE' , 
						'RTT-CHANGE',
						'MIN-RTT',
						'MAX-RTT',
						'INITIAL-RTT',
						'PARALLELISM-CHANGE',
						'MIN-PARALLELISM',
						'MAX-PARALLELISM',
						'ALLOW-IPV6',
						'-IPV6',
						'MAIL-RESULT',
						'SCHEDULED-DATE',
						'TIMING-TEMPLATE',
						'TREAT-ONLINE',
						'PORTS-RANGE',
					],
};


# creates a new Scan Object
# use as it : new Nmap::Scan( Nmap::Main , "target" , scan_id , Nmap::User );
sub new
{
    my $this;
    my $class;
	my $nmap;
	my $target;
	my $user;
	my $opts;
    
    $class 		= 	shift;
	$nmap		=	shift;
	$target		=	shift;
	$user		=	shift;
	$opts		=	shift;


    $this = {};
   	return( undef ) if( ! defined( $target ) );


    $this->{'main'} 	= $nmap 	if( defined $nmap );
    $this->{'target'} 	= $target 	if( defined $target );

	$this->{'id'}		= $target	if( (defined( $opts ) ) && ( $opts->{'METHOD'} eq 'byid' ) );
    $this->{'user'} 	= $user 	if( defined $user );

	$this->{'state'}	= STATE_NEW;

	if( defined( $opts->{TYPE} ) )
	{
		$this->{'type'}	= $opts->{TYPE};
	}
	else
	{
		$this->{'type'}	= TYPE_SINGLE;
	}

    bless( $this , $class );
}


#Create the scan in the database
#Return a negative value if fails
#If it creates successfully the scan, returns the 'id' attribute from the database
sub create
{
	my $self = shift;
	my $query;
	my $stmt;
	my $result;

	#If the user that own the scan does not exists, don't create the scan in the database
	if( $self->{'user'}->getdb != 1 )
	{
		return( -1 );
	}


	#We prepare queries to add the scan in the database
	$query = "INSERT INTO scans ( target ,  uidowner , state , type , create_date ) VALUES( ? , ? , ? , ? , NOW() ) ";
	$stmt = $self->{'main'}->get_db->prepare( $query );
	$stmt->execute( $self->{'target'} , $self->{'user'}->get( 'id' ) , STATE_NEW , $self->{'type'}) or return -1;

	$query = "SELECT id FROM scans WHERE target = ? AND uidowner = ? ORDER BY id DESC";
	$stmt = $self->{'main'}->get_db->prepare( $query );
	$result = $stmt->execute( $self->{'target'} , $self->{'user'}->get( 'id' ) ) or return -1;

	return( -2 ) if( $result < 1 );

	$result = $stmt->fetchrow_arrayref;

	return( $result->[0] );
}



sub save
{
	my $self = shift;
	my $query;
	my $attr;
	my $sth;
	my $key;
	my $state;
	my $value;

	foreach $attr ( qw(type uidowner scanref) )
	{
		next if( ! defined $self->{$attr} );
		$query = "UPDATE scans SET $attr=? WHERE id=?";
		$sth = $self->{'main'}->get_db->prepare( $query );
		$sth->execute( $self->{$attr} , $self->{id} ) or return -1;
	}

	foreach $key ( keys ( %{$self->{'options'}} ) )
	{
		next if( ! defined( $self->{'options'}->{$key}{'state'} ) ); 
		$state = $self->{'options'}->{$key}{'state'};
		$value = $self->{'options'}->{$key}{'value'} if( defined( $self->{'options'}->{$key}{'value'} ) );

		if( $state == OPT_NEW )
		{
			if( defined $value )
			{
				$query = "INSERT INTO scan_opts( sid , optname , value ) VALUES( ? , ? , ? )";
				$sth = $self->{'main'}->get_db->prepare( $query );
				$sth->execute( $self->{id} , $key , $value ) or return -1;
			}
			else
			{
				$query = "INSERT INTO scan_opts( sid , optname ) VALUES( ? , ? )";
				$sth = $self->{'main'}->get_db->prepare( $query );
				$sth->execute( $self->{id} , $key ) or return -1;
			}
		}
		elsif( $state == OPT_UPDATE )
		{
			if( defined $value )
			{
				$query = "UPDATE scan_opts SET value=? WHERE sid=? AND optname=?";
				$sth = $self->{'main'}->get_db->prepare( $query );
				$sth->execute( $value , $self->{id} , $key ) or return -1;
			}
		}
		elsif( $state == OPT_DELETE )
		{
				$query = "DELETE FROM scan_opts WHERE sid=? AND optname=?";
				$sth = $self->{'main'}->get_db->prepare( $query );
				$sth->execute( $self->{id} , $key ) or return -1;
		}


	}

	return 1;
}


sub getdb
{
	my $self;
	my $query;
	my $request;
	my $result;
	my $r;
	my $tmp;
	my @tab;

	$self = shift;
	if( defined $self->{'id'} )
	{
		$query = "SELECT * FROM scans WHERE id=?";
		$request = $self->{'main'}->get_db->prepare( $query ) or return -2;
		$r = $request->execute( $self->{'id'} ) or return -3;
		return 0 if( $r < 1 );
	}
	else
	{
		$query = "SELECT * FROM scans WHERE target=?";
		$request = $self->{'main'}->get_db->prepare( $query ) or return -2;
		$r = $request->execute( $self->{'target'} ) or return -3;
		return 0 if( $r < 1 );
	}

	$result = $request->fetchrow_hashref();

	foreach ( keys %{$result} )
	{
		$self->{$_} = $result->{$_}
	}
	$query = "SELECT * FROM scan_opts WHERE sid=?";
	$request = $self->{'main'}->get_db->prepare( $query );
	$request->execute( $self->{id} ) or return -1;

	$result = $request->fetchall_hashref( 'optname' );
	foreach ( keys %{$result} )
	{
		$self->{options}->{$_}{state} = OPT_NORMAL;
		$self->{options}->{$_}{value} = $result->{$_}{value};
	}

	$query = "SELECT * FROM comments WHERE sid=?";
	$request = $self->{'main'}->get_db->prepare( $query );
	$request->execute( $self->{id} ) or return -1;

	while( $result = $request->fetchrow_hashref )
	{
		$tmp->{'username'} = Nmap::User::get_username_by_uid( $self->{'main'} , $result->{'uid'} );
		$tmp->{'comment'} = stripslashes( $result->{'comment'} );
		$tmp->{'post_date'} = stripslashes( $result->{'post_date'} );
		$tmp->{'cid'} = stripslashes( $result->{'cid'} );
		$tmp->{'sid'} = stripslashes( $result->{'sid'} );
		push( @tab ,$tmp );
	}
	$self->{'comments'} = \@tab;

	return 1;
}


sub get
{
	my $self;
	my $value;
	$self = shift;
	$value = shift;
	return( Nmap::Functions::stripslashes( $self->{$value} ) );
}

sub set
{
	my $self;
	my $key;
	my $value;
	$self = shift;
	$key = shift;
	$value = shift;

	$self->{$key} = Nmap::Functions::addslashes( $value );
}

sub setopt
{
	my $self;
	my $optname;
	my $value;

	$self = shift;
	$optname = shift;
	$value = shift;


	
	if( defined( $self->{options}->{$optname} ) )
	{
		#If the option already exists
		#We update it ONLY if it exists and was not already modified
		if( $self->{options}->{$optname}{state} == OPT_NORMAL )
		{
			$self->{options}->{$optname}{value} = $value if( $value );
			$self->{options}->{$optname}{state} = OPT_UPDATE;
			return( 1 );
		}
		return( -1 );
	}
	else
	{
			$self->{options}->{$optname}{value} = $value if( $value );
			$self->{options}->{$optname}{state} = OPT_NEW;
			return( 1 );
	}
	return( -1 );
}

sub delopt
{
	my $self;
	my $optname;

	$self = shift;
	$optname = shift;

	if( defined( $self->{options}->{$optname} ) )
	{
		$self->{options}->{$optname}{state} = OPT_DELETE;
		return( 1 );
	}

	return( -1 );
}

sub merge_opts
{
	my $self;
	$self = shift;

	foreach ( keys %{$self->{options}} )
	{
		$self->{$_} = $self->{options}->{$_};
	}
}

sub getoptval
{
	my $self;
	my $optname;
	$self = shift;
	$optname = shift;
	return( $self->{options}->{$optname} );
}


sub getall
{
	my ( $main , $tmp , $scan , @ar , @result , $opts , $query , $request , $r );
	$main = shift;
	$opts = shift;

	$opts->{ORDERBY} = 'target' if( ! ( $opts->{ORDERBY} =~ /^(target|create_date|uidowner|type|state)$/ ) );
	$opts->{INVERT} = 1 if( ( $opts->{INVERT} != 1 ) && ( $opts->{INVERT} != 0 ) );


	$query = "SELECT id FROM scans WHERE 1=1";
	$query .= " AND state='".$opts->{STATE}."'" if( defined $opts->{STATE} );
	$query .= " AND target LIKE '%".$opts->{TARGETLIKE}."%'" if( defined $opts->{TARGETLIKE} );
	$query .= " AND ( ( type = ".TYPE_SINGLE." ) OR ( type = ".TYPE_SCHEDULED.") )" if( defined $opts->{'SINGLE-SCHEDULED'} );
	$query .= " AND ( ( type = ".TYPE_PERIODIC." ) OR ( type = ".TYPE_PERIODICSTOPPED.") )" if( defined $opts->{'PERIODIC-PERIODICSTOPPED'} );
	$query .= " AND target='".$opts->{TARGET}."'" if( defined $opts->{TARGET} );
	$query .= " AND type !='".$opts->{NOTYPE}."'" if( defined $opts->{NOTYPE} );
	$query .= " AND state !='".$opts->{NOSTATE}."'" if( defined $opts->{NOSTATE} );
	$query .= " AND type ='".$opts->{TYPE}."'" if( defined $opts->{TYPE} );
	$query .= " AND father ='".$opts->{FATHER}."'" if( defined $opts->{FATHER} );
	$query .= " AND uidowner='".$opts->{OWNER}."'" if( defined $opts->{OWNER} );
	$query .= " AND create_date > '".$opts->{MINCREATEDATE}."'" if( defined $opts->{MINCREATEDATE} );
	$query .= " OFFSET ".$opts->{OFFSET} if( defined $opts->{OFFSET} );
	$query .= " ORDER BY ".$opts->{ORDERBY} if( defined $opts->{ORDERBY} );
	$query .= " DESC " if( ( defined $opts->{INVERT} ) && ( $opts->{INVERT} == 1 ) );
	$query .= " LIMIT ".$opts->{LIMIT} if( defined $opts->{LIMIT} );
	$request = $main->get_db->prepare( $query ) or return( undef );
	$r = $request->execute() or return( undef );
	return undef if( $r <= 0 );

	while( @ar = $request->fetchrow_array )
	{
		$tmp = shift( @ar );
		$scan = new Nmap::Scan( $main , $tmp , undef , { METHOD => 'byid' } );
		next if( $scan->getdb != 1 );
		$scan->{'admin'} = 1 if( ( defined $opts ) && ( $opts->{ADDADMIN} == 1 ) );
		$scan->make_str if( ( defined $opts ) && ( $opts->{MAKESTR} == 1 ) );
		push( @result , $scan );
	}

	return( \@result );
}

sub delete
{
	my ( $self , $query , $stmt );
	$self = shift;

	if( ( $self->{type} == TYPE_PERIODIC ) || ( $self->{type} == TYPE_PERIODICSTOPPED ) )
	{
		use Nmap::Logger;
		$query = "DELETE FROM scan_opts WHERE sid IN ( SELECT id FROM scans WHERE father=?)";
		$stmt = $self->{'main'}->get_db->prepare( $query ) or return -2;
		$stmt->execute( $self->{'id'} ) or return -3;

		$query = "DELETE FROM scans WHERE father=?";
		$stmt = $self->{'main'}->get_db->prepare( $query ) or return -2;
		$stmt->execute( $self->{'id'} ) or return -3;
	}

	$query = "DELETE FROM scan_opts WHERE sid=?";
	$stmt = $self->{'main'}->get_db->prepare( $query ) or return -2;
	$stmt->execute( $self->{'id'} ) or return -3;

	$query = "DELETE FROM scans WHERE id=?";
	$stmt = $self->{'main'}->get_db->prepare( $query ) or return -2;
	$stmt->execute( $self->{'id'} ) or return -3;
	return( 1 );
}


sub get_dir
{
	my $self;
	my $sid;
	my $datadir;
	my( $d , $h , $t );
	$self = shift;
	$datadir = $self->{main}->get_config->get_datadir;
	$sid = $self->get( 'id' );

	use integer;
	
	$d = $sid / 10;
	$h = $sid / 100;
	$t = $sid / 1000;

	no integer;

	return( $datadir . '/' . $t . '/' . $h . '/' . $d . '/' );
}

sub get_txtfile
{
	my( $self , $filename );
	$self = shift;

	$filename = $self->get_dir . "/".$self->{id} . ".txt";

	return( $filename );
}

sub get_xmlfile
{
	my( $self , $filename );
	$self = shift;

	$filename = $self->get_dir . "/".$self->{id} . ".xml";

	return( $filename );
}

sub make_str
{
	my $self;
	$self = shift;

	if( $self->{state} == STATE_FINISHED )
	{
		$self->{statestr} = "Finished";
	}
	if( $self->{state} == STATE_NEW )
	{
		if( $self->{type} == TYPE_SCHEDULED )
		{
			$self->{statestr} = "Scheduled, will be performed";
		}
		else
		{
			$self->{statestr} = "New, waiting for the daemon ...";
		}
	}
	if( $self->{state} == STATE_WIP )
	{
		$self->{statestr} = "In progress, the daemon treat the request ...";
	}
	if( $self->{state} == STATE_INPROGRESS )
	{
		$self->{statestr} = "In progress, the daemon treat the request ...";
	}
	if( $self->{state} == STATE_EXECERROR )
	{
		$self->{statestr} = "Error when treat the request";
	}



	if( $self->{type} == TYPE_SINGLE )
	{
		$self->{typestr} = "At once";
	}
	elsif( $self->{type} == TYPE_SCHEDULED )
	{
		$self->{typestr} = "Scheduled";
	}
	elsif( $self->{type} == TYPE_PERIODIC )
	{
		$self->{typestr} = "Periodic";
		$self->{statestr} = "Periodic, scans are performed in time";
		$self->{'periodic_active'} = "yes";
		$self->{active} = 1;
		#Show periodic interval in hours
		$self->{periodic_interval} = ( $self->{'options'}{'PERIODIC-INTERVAL'}{value} / 3600 );
	}
	elsif( $self->{type} == TYPE_INHERITED )
	{
		$self->{INHERITED} = 1;
		$self->{typestr} = "Inherited";
	}
	elsif( $self->{type} == TYPE_PERIODICSTOPPED )
	{
		$self->{typestr} = "Periodic (stopped)";
		$self->{statestr} = "Periodic, scans are stopped";
		$self->{'periodic_active'} = "no";
		$self->{idle} = 1;
		#Show periodic interval in hour
		$self->{periodic_interval} = ( $self->{'options'}{'PERIODIC-INTERVAL'}{value} / 3600 );
	}

	if( ! defined( $self->{scan_date} ) )

	{
		$self->{scan_date} = "Unknow";
	}

	$self->{'create_date'} =~ s/\..*//g;
	$self->{'scan_date'} =~ s/\..*//g;


	if( defined( $self->{options}->{'TIMING-TEMPLATE'} ) )
	{
		if( $self->{options}->{'TIMING-TEMPLATE'}{'value'} == 1 )
		{
			$self->{'TIMING-TEMPLATE-VALUE'} = "Very Heavy";
		}
		elsif( $self->{options}->{'TIMING-TEMPLATE'}{'value'} == 2 )
		{
			$self->{'TIMING-TEMPLATE-VALUE'} = "Heavy";
		}
		elsif( $self->{options}->{'TIMING-TEMPLATE'}{'value'} == 3 )
		{
			$self->{'TIMING-TEMPLATE-VALUE'} = "Normal";
		}
		elsif( $self->{options}->{'TIMING-TEMPLATE'}{'value'} == 4 )
		{
			$self->{'TIMING-TEMPLATE-VALUE'} = "Fast";
		}
		elsif( $self->{options}->{'TIMING-TEMPLATE'}{'value'} == 5 )
		{
			$self->{'TIMING-TEMPLATE-VALUE'} = "Very fast";
		}
		elsif( $self->{options}->{'TIMING-TEMPLATE'}{'value'} == 1 )
		{
			$self->{'TIMING-TEMPLATE-VALUE'} = "Very Heavy";
		}
	}

	if( $#{$self->{'comments'}} > -1 )
	{
		$self->{'HAVE-COMMENTS'} = 1;
	}

	if( defined( $self->{options}{'PORTS-RANGE'} ) )
	{
		$self->{'PORTS-RANGE-VALUE'} = $self->{options}{'PORTS-RANGE'}{value};
	}

}

sub is_ip
{
	my( $self , $tmp );
	$self = shift;
	$tmp = $self->{target};
	return 0 if( ! defined( $tmp  ) );

	if( $tmp =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/ )
	{
		return 1;
	}
	return 0;
}



sub is_subnet
{
	my( $self , $tmp );
	$self = shift;
	$tmp = $self->{target};
	return 0 if( ! defined( $tmp  ) );

	if( $tmp =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\/\d{1,2}$/ )
	{
		return 1;
	}
	return 0;
}



sub is_hostname
{
	my( $self , $tmp );
	$self = shift;
	$tmp = $self->{target};
	return 0 if( ! defined( $tmp  ) );

	if( $tmp =~ /^([a-zA-Z0-9]+\-{0,1}[a-zA-Z0-9]+\.{0,1})+$/ )
	{
		return 1;
	}
	return 0;
}

sub addpost
{
	my $uid;
	my $title;
	my $str;
	my $self;
	my ( $query , $request , $result );

	$self = shift;
	$title = shift;
	$str = shift;
	$uid = shift;

	return -1 if( 	(	 ! defined( $title ) ) ||
					( ! defined( $str ) ) ||
					( ! defined( $uid ) )
				);
	
	$query = "INSERT INTO comments( uid , sid , post_date , title , comment) VALUES( ? , ? , NOW() , ? , ? )";
	$request = $self->{main}->get_db->prepare( $query ) or return -2;
	$request->execute( $uid , $self->{'id'} , addslashes( $title ) , addslashes( $str ) ) or return -3;
	return 1;
}
sub delpost
{
	my $cid;
	my $self;
	my ( $query , $request , $result );

	$self = shift;
	$cid = shift;

	$query = "DELETE FROM comments WHERE cid=? and sid=?";
	$request = $self->{main}->get_db->prepare( $query ) or return -2;
	$request->execute( $cid , $self->{'id'} ) or return -3;
	return 1;
}


1;

__END__
=head1 SYNOPSIS

Nmap::Scan handle scans for nmap-cgi program

=head1 DESCRIPTION

This module is an object which handle the scans of nmap-cgi. This module is OO oriented.

=head1 METHODS

=head2 new

Make a new instance of this object. The first param is the username. You could give an gid as second argument, but int this case, the third argument should be a hash like this : { METHOD => 'id' }

=head2 create()

Create the object in the database. Returns 1 if success.

=head2 save()

Save the current objet in the database. Save options, rights, etc ...

=head2 getdb()

Get the scan from the database. Returns 1 if success.

=head2 opt( $optname )

Returns 1 if this scan has the the option $optname

=head2 setopt( $optname , $value )

Set the option $optname to $value.

=head2 getoptval( $optname )

Get the value of the option $optname

=head2 set( $attr , $value )

Set the attribute $attr to $value

=head2 get( $attr )

The the value of the attribute $attr

=head2 delopt( $optname )

Remove the option $optname

=head2 getall()

Get all scans that matches with the options. The first argument is a reference to a Nmap::Main instance. The second is a hash which contains options.
Options are hashkey, options value are hashvalues.
There are the list of hashkey :

	* MINCREATEDATE - give a string to search all groups that seems have this groupname

	* TARGET - Get all scans that have a target equal to hashvalue
	
	* NOTYPE - Get all the scans that don't have a type like hashvalue
	
	* NOSTATE - Get all scans that don't have a state like hashvalue
	
	* TARGETLIKE - Get all scans that target are like hashvalue
	
	* OFFSET 
	
	* LIMIT
	
	* ORDERBY
	
	* DESC
	
	* LIMIT
	
	* OWNERBY - All scans that are owned by hashvalue uid
	
	* FATHER - Take all the scans that have the hashvalue scanid as father.


=head2 delete

Delete the scan from database. Returns 1 if success

=head2 is_hostname()

Returns 1 if the target is a hostname

=head2 is_ip()

Returns 1 if the target is an IP

=head2 is_subnet()

Returns 1 if the target is a subnet

=head2 make_str

Add some strings to print relevant informations on form. Used when print scans with the cgi part.

=head2 get_xmlfile()

Get the path of the xmlfile of this scan.

=head2 get_txtfile()

Get the path of the textfile which contains the result of the scan.

=head2 addpost( $title , $str , $uid )

Add a post to this scan. The first argument is the title of the scan. The second if the content of the comment. The third parameter is the userid.
Returns 1 if success.

=head2 delpost( $cid )

Delete a post from this scan. The first argument is the comment-id of the scan. Returns 1 if success.

=head1 SEE ALSO

Nmap::Main , Nmap::User , Nmap::Group

=head1 AUTHOR

Julien Delange <julien at gunnm dot org>

=head1 COPYRIGHT

Julien Delange <julien at gunnm dot org>

