#!/usr/bin/perl -w

#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


use Crypt::PasswdMD5; #Used for encryption

package Nmap::User;

use strict;
use DBI;
use Socket;
use Nmap::Functions;
use Email::Valid;

use constant
{
	STATE_NEW	=> 0,
	STATE_CONFIRMED => 1,
	STATE_ADMIN => 2
};


###################
# Create a new User
# Should be used as : new Nmap::User( Nmap::Main , "username" );
# Ex: $main = init Nmap::Main;
#     $user = new Nmap::User( $main , "username" );
sub new
{
    my $this;
    my $class;
	my $nmap;
	my $username;
	my $opts;
    
    $class 		= 	shift;
	$nmap		=	shift;
	$username	=	shift;
	$opts		=	shift;


    $this = {};
    
   	return( undef ) if( ! defined( $username ) );

    $this->{'main'} = $nmap;
    $this->{'username'} = $username;

	if( defined $opts )
	{
		$this->{id} = $username if( ( defined ( $opts->{METHOD} )  ) && ( $opts->{METHOD} eq 'id' ) );
	}
    
    bless( $this , $class );
}


#####################
# Create the user in the database
# Returns 1 if successfull, negative value otherwise
sub create
{
	my $self = shift;
	my $query;
	my $stmt;
	my $r;

	$query = "INSERT INTO users( username , state ) VALUES( ? , ?) ";
	$stmt = $self->{'main'}->get_db->prepare( $query );
	$stmt->execute( $self->{'username'} , STATE_NEW ) or return -1;

	$r = $self->save;

	return -1 if( $r != 1 );
	return 1;
}


#####################
# Save user preferences and options 
# in the database
sub save
{
	my $self = shift;
	my $query;
	my $attr;
	my $sth;

	foreach $attr ( qw(firstname lastname mail address passhash chash state gid theme) )
	{
		next if( ! defined $self->{$attr} );
		$query = "UPDATE users SET $attr=? WHERE username=?";
		$sth = $self->{'main'}->get_db->prepare( $query ) or return( -1 );
		$sth->execute( $self->{$attr} , $self->{username} ) or return -1;
	}

	return 1;
}


######################
# Get the user from the database
# ex: $user->getdb
sub getdb
{
	my $self;
	my $query;
	my $request;
	my $result;
	my $r;

	$self = shift;

	if( defined $self->{id} )
	{
		$query = "SELECT * FROM users WHERE id=?";
		$request = $self->{'main'}->get_db->prepare( $query ) or return -2;
		$r = $request->execute( $self->{id} ) or return -3;
	}
	else
	{
		$query = "SELECT * FROM users WHERE username=?";
		$request = $self->{'main'}->get_db->prepare( $query ) or return -2;
		$r = $request->execute( $self->{'username'} ) or return -3;
	}
	
	return 0 if( $r < 1 );

	$result = $request->fetchrow_hashref();

	foreach ( keys %{$result} )
	{
		$self->{$_} = Nmap::Functions::stripslashes( $result->{$_} );
	}

	return 1;
}


######################
# Get an attribute for the user
# Example : $user->get( 'firstname' );
sub get
{
	my $self;
	my $value;
	$self = shift;
	$value = shift;
	return( Nmap::Functions::stripslashes( $self->{$value} ) );
}


#####################
# Set an attribute for the user
# Example : $user->set( 'firstname' , 'Julien' );
sub set
{
	my $self;
	my $key;
	my $value;
	$self = shift;
	$key = shift;
	$value = shift;

	if( $key eq 'password' )
	{
		$self->{'passhash'} = encrypt( $value );
	}
	elsif( $key eq 'mail' )
	{
		#Put the adress only if it's valid
		if( Email::Valid->address( $value ) )
		{
			$self->{'mail'} = $value;
		}
	}
	else
	{
		$self->{$key} = Nmap::Functions::addslashes( $value );
	}
}

sub salt
{
	my $length;
	$length = 8;
	return join "", ('.', '/', 0..9, 'A'..'Z', 'a'..'z')[map {rand 64} (1..$length)];
}

sub encrypt
{
	my $password;
	my $crypted;

	$password = shift;

    return( Crypt::PasswdMD5::unix_md5_crypt( $password , salt() ) );
}

sub delete
{
	my ( $self , $query , $stmt );
	$self = shift;

	$query = "DELETE FROM userips WHERE userid=?";
	$stmt = $self->{'main'}->get_db->prepare( $query ) or return -2;
	$stmt->execute( $self->{'id'} ) or return -3;

	$query = "DELETE FROM users WHERE id=?";
	$stmt = $self->{'main'}->get_db->prepare( $query ) or return -2;
	$stmt->execute( $self->{'id'} ) or return -3;
}


sub get_id
{
	my $self = shift;
	return( $self->{'id'} );
}

sub getall
{
	my ( $main , $tmp , $user , @ar , @result , $opts , $query , $request );
	$main = shift;
	$opts = shift;
	
	$opts->{ORDERBY} = 'username' if( ! ( $opts->{ORDERBY} =~ /^(username|mail|lastname|firstname)$/ ) );
	$opts->{INVERT} = 1 if( ( $opts->{INVERT} != 1 ) && ( $opts->{INVERT} != 0 ) );

	$query = "SELECT username FROM users WHERE 1=1";
	$query .= " AND state = ".$opts->{STATE} if( defined $opts->{STATE} );
	$query .= " AND username LIKE '%".$opts->{USERNAME}."%'" if( defined $opts->{USERNAME} );
	$query .= " AND chash = ".$opts->{CHASH} if( defined $opts->{CHASH} );
	$query .= " ORDER BY ".$opts->{ORDERBY} if( defined $opts->{ORDERBY} );
	$query .= " DESC " if( ( defined $opts->{INVERT} ) && ( $opts->{INVERT} == 1 ) );
	$query .= " OFFSET ".$opts->{OFFSET} if( defined $opts->{OFFSET} );
	$query .= " LIMIT ".$opts->{LIMIT} if( defined $opts->{LIMIT} );
	$request = $main->get_db->prepare( $query ) or return -2;
	$request->execute() or return -3;

	while( @ar = $request->fetchrow_array )
	{
		$tmp = shift( @ar );
		$user = new Nmap::User( $main , $tmp );
		next if( $user->getdb != 1 );
		push( @result , $user);
	}
	return( \@result );
}

sub pwdmatch
{
	my $self;
	my $pwd;
	my $tmp;

	$self = shift;
	$pwd = shift;
	if( $self->{'passhash'} eq Crypt::PasswdMD5::unix_md5_crypt($pwd, $self->{'passhash'}) )
	{
		return 1;
	}
	return 0;
}

sub get_group
{
	my $self;
	my $group;

	$self = shift;

	if( ( ! ( defined $self->{'gid'} ) ) || ( $self->{gid} eq '' ) )
	{
		$group = Nmap::Group::getall( $self->{'main'} , { DEFAULT => 1 } );
		return undef if( ! ( defined $group ) );
		$group = shift( @{$group} );
	}
	else
	{
		$group = new Nmap::Group( $self->{main} , $self->{gid} , { METHOD => 'id' } );
		return undef if( ! defined( $group ) );
		return undef if( $group->getdb != 1 );
	}
	return $group;
}

sub is_admin
{
	my $self;
	$self = shift;
	return( 1 ) if( ( defined( $self->{state} ) ) && ( $self->{state} == 2 ) );
	return( 0 );
}

sub get_theme
{
	my $self;
	$self = shift;

	if( ( defined( $self->{theme} ) ) && ( $self->{theme} ne '' ) )
	{
		return( $self->{theme} );
	}

	#If no theme are stored in used preferences, we return default theme from the config file
	return( $self->{'main'}->get_config->get_default_theme );
}


sub fetch_ips
{
	my( $query , $request , $stmt , $self  , @row);

	$self = shift;

	$query = "SELECT adr  FROM userips WHERE userid=?";
	$stmt = $self->{'main'}->get_db->prepare( $query ) or return undef;
	$stmt->execute( $self->{'id'} ) or return undef;

	while( @row = $stmt->fetchrow_array )
	{
		push( @{$self->{'ips'}} , { IP => $row[0] , ADR => gethostbyaddr( $row[0] , AF_INET ) } );
	}
}

sub get_username_by_uid
{
	my $nmap;
	my $uid;
	my ( $query , $request , $result );

	$nmap = shift;
	$uid = shift;

	$query = "SELECT username FROM users WHERE id=?";
	$request = $nmap->get_db->prepare( $query ) or return -2;
	$request->execute( $uid ) or return -3;
	
	$result = $request->fetchrow_hashref();

	return( $result->{'username'} );
}

1;
__END__
=head1 SYNOPSIS

Nmap::User handle users for nmap-cgi program

=head1 DESCRIPTION

This module is an object which handle the users of nmap-cgi. This module is OO oriented.

=head1 METHODS

=head2 new

Create a new instance of this object. The first argument is a instance of Nmap::Main object. The second is the username. 
The third is optionnal and can be a hash which contains options. 
If the options contains the hashkey called METHOD with a value of 'id', the first argument will be taken as argument.

=head2 create()

Create the object. If it is successfull, returns 1. Returns another value if fails.

Returns the subopen configuration directive. Returns undef if not defined.

=head2 save()

Save the current objet in the database. Returns 1 if success

=head2 getdb()

Get the user grom the database. If the option in the new method tell search by id, it will search the user by id. Otherwise, it search by username.
Return 1 if the user exists.

=head2 get( $attr )

Returns the value of attribute $attr. Return undef is the attribute does not exist.

=head2 set( $attr , $attrval )

Set the value of attribute $attr to $attrval. Be careful, when you invoke the set method, it does not change values in the database. Invoke save method to perform that.

=head2 get_theme()

Return the current theme for this user.

=head2 fetch_ips()

Returns an array with all ip from this user

=head2 get_group()

Return the Nmap::Group instance that match with this user.

=head2 pwdmatch( $pass )

The argument is a string. Returns 1 if the password of the current user is that string. Return false otherwise.

=head2 get_theme()

Returns an arrayref where the array contains the name of the available themes declared in the config file.

=head2 getall()

As all getall methods, it returns all users that matchs with option. The first argument must be a reference to a Nmap::Main object.
The second param is a hash. Each hashkey is an option and the hashvalue if the value of the option. There is the list of options


=head1 EXAMPLE
	
$main = init Nmap::Main;

$user = new Nmap::User( $main , "foobar" );

$user->create;

=head1 SEE ALSO

Nmap::Main

=head1 AUTHOR

Julien Delange <julien at gunnm dot org>

=head1 COPYRIGHT

Julien Delange <julien at gunnm dot org>

