/*
Copyright (c) Nmap project and its contributors
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions 
are met:

 1. Redistributions of source code must retain the above copyright 
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in 
    the documentation and/or other materials provided with the 
    distribution.
 3. Neither the name of Nmap nor the names of its contributors 
    may be used to endorse or promote products derived from this 
    software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.
*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include <config.h>
#include <request.h>
#include <scan.h>
#include <debug.h>

extern config_t* config;


/* Mail the result to the user
 * Give the id of the scan in parameter
 * and send a mail to the scan owner with the result files
 */
int mail_result( const int scanid )
{
	char* adr;
	char* txtfilename;
	char* xmlfilename;
	char* target;
	char* cmd;
	char* from;
	char* args[3];
	int pip[2];
	int pid;
	int sav;
	int fd;
	char tmp[1024];

	adr = get_mail( scanid );
	txtfilename = find_txtfilename( scanid );
	xmlfilename = find_xmlfilename( scanid );
	target = get_target( scanid );
	cmd = config->sendmail;
	from = config->mailfrom;

	if( cmd == NULL )
		return( -1 );
	if( adr == NULL )
		return( -1 );
	if( xmlfilename == NULL )
		return( -1 );
	if( txtfilename == NULL )
		return( -1 );
	if( from == NULL )
		return( -1 );

	args[0] = cmd;
	args[1] = adr;
	args[2] = NULL;

	if( pipe( pip ) != 0 )
	{
		free( txtfilename );
		free( xmlfilename );
		free( adr );
		return( -2 );
	}

	/*
	 * Create another process.
	 * This new process will send the mail and current process
	 * will print the content of the mail to his child.
	 */
	pid = fork();
	if( pid == -1 )
	{
		return( -2 );
	}
	if( pid == 0 )
	{
		/*
		 * Child process to the send the mail
		 * We perform following operations
		 * 			o Replace out STDIN file descriptor
		 * 			  with STDOUT of our father. We use the pipe(3) call to do it
		 * 			o Start sendmail(1) to send the mail
		 */
		close( pip[1] );
		dup2(  pip[0] , STDIN_FILENO );
		close( pip[0] );
		execv( cmd , args );
		exit( EXIT_FAILURE ); /* should never reach this point */
	}

	/*
	 * Replace STDOUT of the current process with
	 * STDIN of our child process
	 */
	sav = dup( STDOUT_FILENO );
	close( pip[0] );
	dup2( pip[1] , STDOUT_FILENO );
	close( pip[1] );

	/*
	 * Send mail headers according to scan 
	 * informations
	 */
	fprintf( stdout , "From: %s\n" , from );
	fprintf( stdout , "Subject: Scan result for host %s\n" , target );
	fprintf( stdout , "MIME-Version: 1.0\n");
	fprintf( stdout , "Content-Type: multipart/mixed; boundary=\"3MwIy2ne0vdjdPXF\"\n" );

	fprintf( stdout , "\n" );
	fprintf( stdout , "\n" );

	/*
	 * Send the content of the mail
	 */
	fprintf( stdout , "--3MwIy2ne0vdjdPXF\n");
	fprintf( stdout , "Content-Type: text/plain; charset=utf-8\n");
	fprintf( stdout , "Content-Disposition: inline\n");

	fprintf( stdout , "Hello,\n");
	fprintf( stdout , "You will find as attachment results for the scan on the following target(s)\n");
	fprintf( stdout , "%s\n" , target);
	fprintf( stdout , "The result are available as normal output and XML output\n" );
	fprintf( stdout , "\n");


	/*
	 * Open the XML file and read it
	 * send it as attachment in the mail
	 */
	fd = open( xmlfilename , O_RDONLY);
	if( fd > 0 )
	{
		fprintf( stdout , "\n" );
		fprintf( stdout , "--3MwIy2ne0vdjdPXF\n");
		fprintf( stdout , "Content-Type: text/xml\n" );
		fprintf( stdout , "Content-Disposition: attachment; filename=\"scan-result.xml\"\n" );
		fprintf( stdout , "\n" );
		memset( tmp , '\0' , 1024 );
		while( read( fd , tmp , 1023 ) > 0 )
		{
			fprintf( stdout , tmp );
			memset( tmp , '\0' , 1024 );
		}
		close( fd );
		fprintf( stdout , "\n" );
	}

	/* 
	 * Open the TXT result file
	 * and send it to the user as mail attachment
	 */
	fd = open( txtfilename , O_RDONLY);
	if( fd > 0 )
	{
		fprintf( stdout , "\n" );
		fprintf( stdout , "--3MwIy2ne0vdjdPXF\n");
		fprintf( stdout , "Content-Type: text/txt\n" );
		fprintf( stdout , "Content-Disposition: attachment; filename=\"scan-result.txt\"\n" );
		fprintf( stdout , "\n" );
		memset( tmp , '\0' , 1024 );
		while( read( fd , tmp , 1023 ) > 0 )
		{
			fprintf( stdout , tmp );
			memset( tmp , '\0' , 1024 );
		}
		close( fd );
		fprintf( stdout , "\n" );
	}


	/*
	 * Print the end of the mail
	 */
	fprintf( stdout , "--3MwIy2ne0vdjdPXF--\n");
	fprintf( stdout , "%c\n" , EOF );
	fflush( NULL );

	close( STDOUT_FILENO );


	/* 
	 * We made a dup2() to replace stdout
	 * We restore it now
	 */
	dup2( sav , STDOUT_FILENO );
	close( sav );
	fflush( NULL );


	free( adr );
	free( txtfilename );
	free( xmlfilename );
	free( target );
	return( 0 );
}




/* Send a mail to the owner of the scan
 * to notify him an error
 */
int mail_error( const int scanid )
{
	char* adr;
	char* target;
	char* cmd;
	char* from;
	char* args[3];
	int pip[2];
	int pid;
	int sav;

	adr = get_mail( scanid );
	target = get_target( scanid );
	cmd = config->sendmail;
	from = config->mailfrom;

	if( cmd == NULL )
		return( -1 );
	if( adr == NULL )
		return( -1 );
	if( from == NULL )
		return( -1 );

	args[0] = cmd;
	args[1] = adr;
	args[2] = NULL;

	if( pipe( pip ) != 0 )
	{
		free( adr );
		return( -2 );
	}

	/*
	 * Create another process.
	 * This new process will send the mail and current process
	 * will print the content of the mail to his child.
	 */
	pid = fork();
	if( pid == -1 )
	{
		return( -2 );
	}
	if( pid == 0 )
	{
		/*
		 * Child process to the send the mail
		 * We perform following operations
		 * 			o Replace out STDIN file descriptor
		 * 			  with STDOUT of our father. We use the pipe(3) call to do it
		 * 			o Start sendmail(1) to send the mail
		 */
		close( pip[1] );
		dup2(  pip[0] , STDIN_FILENO );
		close( pip[0] );
		execv( cmd , args );
		exit( EXIT_FAILURE ); /* should never reach this point */
	}

	/*
	 * Replace STDOUT of the current process with
	 * STDIN of our child process
	 */
	sav = dup( STDOUT_FILENO );
	close( pip[0] );
	dup2( pip[1] , STDOUT_FILENO );
	close( pip[1] );

	/*
	 * Send mail headers according to scan 
	 * informations
	 */
	fprintf( stdout , "From: %s\n" , from );
	fprintf( stdout , "Subject: Error when scanning %s\n" , target );
	fprintf( stdout , "MIME-Version: 1.0\n");
	fprintf( stdout , "Content-Type: text-plain; charset=\"utf-8\"\n" );

	fprintf( stdout , "\n" );
	fprintf( stdout , "\n" );

	/*
	 * Send the content of the mail
	 */
	fprintf( stdout , "Hello,\n");
	fprintf( stdout , "An error occured when the system tried to perform a scan\n");
	fprintf( stdout , "It cannot be performed. Please contact the administrator\n");
	fprintf( stdout , "\n");

	fflush( NULL );

	close( STDOUT_FILENO );


	/* 
	 * We made a dup2() to replace stdout
	 * We restore it now
	 */
	dup2( sav , STDOUT_FILENO );
	close( sav );
	fflush( NULL );


	free( adr );
	free( target );
	return( 0 );
}
