/*
Copyright (c) Nmap project and its contributors
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions 
are met:

 1. Redistributions of source code must retain the above copyright 
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in 
    the documentation and/or other materials provided with the 
    distribution.
 3. Neither the name of Nmap nor the names of its contributors 
    may be used to endorse or promote products derived from this 
    software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.
*/
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#include <libpq-fe.h>

#include <config.h>
#include <periodic.h>
#include <debug.h>
#include <scan.h>
#include <request.h>
#include <db.h>
#include <logger.h>
#include <mail.h>


config_t* 	config;
PGconn*		dbconn;
int			nullfd;

/* All that will be done before exit the program :
 * 	- close the connection to the database
 * 	- free structures
 * 	- ...
 */
void exit_program() 
{
	dbclose( dbconn );
	free_config( config );

	nmapcloselog();
	if( close( nullfd ) != 0 )
	{
		perror( "close" );
	}

	exit( EXIT_SUCCESS );
}

/*
 * Signal handler, just treat SIGINT (CTRL + C)
 * and SIGQUIT to quit properly when these
 * signal are sent
 */
void sig_handler( int sig )
{
	switch( sig )
	{
		case SIGINT:
		case SIGQUIT:
			exit_program();
			break;
	}
}

/*
 * Youhou, main function !
 */
int main( int argc , char* argv[] )
{
	int n;
	struct sigaction action;
	sigset_t	 sig;

	/*
	 * Parse the config file and return an config_t var
	 * if the car is NULL, the config is not good and we
	 * should exit
	 */
	config = (config_t*) parse_conf( CONFIG_FILE );
	if( config == NULL )
	{
		fprintf( stderr , "Unable to init config\n" );
		exit( EXIT_FAILURE );
	}

	/*
	 * Perform database connection and check it
	 * Exit if the database connection is not good
	 */
	dbconn = (PGconn*) db_connect( config );
	if( ! check_conn( dbconn ) )
	{
		fprintf( stderr , "Unable to connect to the database\n" );
		exit( EXIT_FAILURE );
	}


	/*
	 * Init logfile
	 */
	if( initlog( config ) != 1 )
	{
		fprintf( stderr , "Unable to init logfile\n" );
		exit( EXIT_FAILURE );
	}

	/*
	 * Replace SIGINT and SIGQUIT
	 */
	sigemptyset( &sig );
	action.sa_mask = sig;
	action.sa_flags = 0;
	action.sa_handler = sig_handler;
	sigaction( SIGINT  , &action , 0 );
	sigaction( SIGQUIT  , &action , 0 );

	init_scans();	
	nullfd = open( "/dev/null" , O_RDWR );

	
	/*
	 * All the time, program do these operations
	 * 	o Check if some periodics scans needs new scan
	 * 	o Perform single, scheduled and inherited scans
	 * 	o sleep 1 sec
	 */
	while( 1 )
	{
		new_inherited();

		/*
		 * Run scans only if we can
		 */
		if( can_run() == 1 )
		{
			n = perform_scans();
			if( n == 0 )
				DEBUGMSG( "No scan should be performed" );
			if( n < 0 )
				DEBUGMSG( "Error when try to perform scan" );
		}
		else
		{
			DEBUGMSG( "Don't execute scans, pool is full" );
		}
		wait_scans();
		sleep( 1 );
	}
	
	return( EXIT_SUCCESS ); /* Never reach this point */
}
